class ActiveFixture(object):

    def __init__(self, fixture):
        super(ActiveFixture, self).__init__()
        self.fixture = fixture
        self.id = fixture.info.id
        self.name = fixture.info.name
        self._cleanups = []

    def add_cleanup(self, cleanup):
        self._cleanups.append(cleanup)

    def do_cleanups(self):
        while self._cleanups:
            cleanup = self._cleanups.pop()
            cleanup()



import itertools
from contextlib import contextmanager

from .element import Element
from .function import Function

_SUCCESS, _FAIL, _ERROR, _INTERRUPT, _SKIP, _NOT_RUN = ['SUCCESS', 'FAIL', 'ERROR', 'INTERRUPT', 'SKIP', 'NOT_RUN']

class Test(Function, Element):

    def __init__(self, suite, file):
        super(Test, self).__init__(suite)
        self.file = file
        self._expected_result = _SUCCESS
        self._selected = True
        self.when_run = WhenRunHelper(self)
        self._repetitions = 1

    def repeat(self, num_repetitions):
        self.add_decorator('slash.repeat({0})'.format(num_repetitions))
        self.expect_repetition(num_repetitions)

    def expect_repetition(self, num_repetitions):
        self._repetitions *= num_repetitions

    def get_num_expected_repetitions(self):
        return self._repetitions

    def add_cleanup(self, **kw):
        return self.add_deferred_event('slash.add_cleanup', name='test_cleanup', **kw)

    def expect_failure(self):
        self._expect(_FAIL)

    def expect_error(self):
        self._expect(_ERROR)

    def expect_interruption(self):
        self._expect(_INTERRUPT)

    def expect_skip(self):
        self._expect(_SKIP)

    def expect_not_run(self):
        self._expect(_NOT_RUN)

    def expect_deselect(self):
        self._selected = False

    def is_selected(self):
        return self._selected

    def _expect(self, expected_result):
        if self._expected_result != _SUCCESS:
            raise NotImplementedError() # pragma: no cover
        self._expected_result = expected_result

    def get_expected_result(self):
        return self._expected_result

    def is_method_test(self):
        raise NotImplementedError() # pragma: no cover

    def is_function_test(self):
        raise NotImplementedError() # pragma: no cover

    def _write_prologue(self, code_formatter):
        self._write_event(code_formatter, 'test_start')
        super(Test, self)._write_prologue(code_formatter)

    def _write_epilogue(self, code_formatter):
        self._write_event(code_formatter, 'test_end')
        super(Test, self)._write_epilogue(code_formatter)

    def _write_cleanups(self, code_formatter):
        for index, cleanup_event in enumerate(self._cleanups, 1):
            code_formatter.writeln('@slash.add_cleanup')
            code_formatter.writeln('def _cleanup{0}():'.format(index))
            with code_formatter.indented():
                code_formatter.writeln('__ut__.events.add({0!r})'.format(cleanup_event))

    def _get_function_name(self):
        return 'test_{0}'.format(self.id)


class WhenRunHelper(object):

    def __init__(self, test):
        super(WhenRunHelper, self).__init__()
        self.test = test

    def raise_exception(self):
        self.test.append_line('raise Exception("Test exception")')
        self.test.expect_error()

    error = raise_exception

    def fail(self):
        self.test.append_line('assert False')
        self.test.expect_failure()

    def interrupt(self):
        self.test.append_line('raise KeyboardInterrupt()')
        self.test.expect_interruption()

    def skip(self):
        self.test.append_line('slash.skip_test("reason")')
        self.test.expect_skip()



import gossip.exceptions
import gossip.hooks
from slash import hooks

from .utils import TestCase


class CustomHooksTest(TestCase):

    def setUp(self):
        super(CustomHooksTest, self).setUp()
        self.hook_name = "some_custom_hook"
        self.hook = hooks.add_custom_hook(self.hook_name)

    def tearDown(self):
        hooks.remove_custom_hook(self.hook_name)
        self.assertFalse(hasattr(hooks, self.hook_name))
        super(CustomHooksTest, self).tearDown()

    def test_hooks_are_globally_available_through_hooks_module(self):
        self.assertIsInstance(hooks.some_custom_hook, gossip.hooks.Hook)
        self.assertIs(hooks.some_custom_hook, self.hook)

    def test_ensure_custom_hook(self):
        self.assertIs(hooks.ensure_custom_hook(self.hook_name), self.hook)
        new_hook = hooks.ensure_custom_hook("new_custom_hook")
        self.addCleanup(hooks.remove_custom_hook, "new_custom_hook")
        self.assertIs(new_hook, hooks.new_custom_hook)

    def test_hooks_appear_in_get_all_hooks(self):
        all_hooks = dict(hooks.get_all_hooks())
        self.assertIs(all_hooks[self.hook_name], self.hook)

    def test_cannot_reinstall_hook_twice(self):
        with self.assertRaises(gossip.exceptions.NameAlreadyUsed):
            hooks.add_custom_hook(self.hook_name)

    def test_cannot_install_default_hooks(self):
        with self.assertRaises(gossip.exceptions.NameAlreadyUsed):
            hooks.add_custom_hook("test_start")



import code
import sys

import pytest
import slash
from slash.utils import interactive
from slash import start_interactive_shell


def test_interactive_test(forge, suite, checkpoint):

    def _interact(*_, **__):
        assert slash.context.session.scope_manager.get_current_stack() == ['session', 'module', 'test']
        checkpoint()

    forge.replace_with(interactive, '_interact', _interact)

    summary = suite.run(additional_args=['-i'])
    assert checkpoint.called

    result = next(summary.session.results.iter_test_results())
    assert repr(result.test_metadata) == '<Interactive>'


@pytest.mark.parametrize('with_session', [True, False])
def test_interactive_scope(forge, with_session):
    interact_mock = forge.replace(interactive, '_interact')

    interact_mock({'x': 2, 'y': 3})
    forge.replay()

    if with_session:
        with slash.Session():
            slash.g.x = 2
            start_interactive_shell(y=3)
    else:
        start_interactive_shell(x=2, y=3)



def test_repeating_test(suite, suite_test):
    num_repetitions = 5
    suite_test.repeat(num_repetitions)

    summary = suite.run()
    results = summary.get_all_results_for_test(suite_test)
    assert len(results) == num_repetitions
    assert len(set(result.test_metadata.id for result in results)) == len(results)


def test_global_repeat(suite, config_override):
    num_repetitions = 5
    config_override('run.repeat_each', num_repetitions)
    for test in suite:
        test.expect_repetition(num_repetitions)
    assert suite.run().ok()



from slash.utils.exception_mark import ExceptionMarker

def test_marker():
    exc = AttributeError()
    marker = ExceptionMarker('mark')
    assert not marker.is_exception_marked(exc)
    marker.mark_exception(exc)
    assert marker.is_exception_marked(exc)




import pytest
import gossip
import slash
import slash.runner
from slash import exception_handling, Session
from slash.loader import Loader

from .utils import TestCase


def test_test_cleanups_happen_before_fixture_cleanups(suite, suite_test):
    fixture = suite.slashconf.add_fixture()
    suite_test.depend_on_fixture(fixture)
    fixture_cleanup = fixture.add_cleanup()
    test_cleanup = suite_test.add_cleanup()
    events = suite.run().events

    assert events[fixture_cleanup].timestamp > events[test_cleanup].timestamp


def test_cannot_add_cleanup_without_active_session():
    with pytest.raises(RuntimeError):
        slash.add_cleanup(lambda: None)


def test_cleanups(suite, suite_test):

    cleanup = suite_test.add_cleanup()

    summary = suite.run()

    assert summary.events[cleanup]


def test_cleanup_args_kwargs_deprecated():
    with slash.Session() as s:
        slash.add_cleanup(lambda: None, "arg1", arg2=1)
    [w] = s.warnings
    assert 'deprecated' in str(w).lower()


def test_cleanup_ordering(suite, suite_test):
    cleanup1 = suite_test.add_cleanup()
    cleanup2 = suite_test.add_cleanup()
    events = suite.run().events

    assert events[cleanup1].timestamp > events[cleanup2].timestamp


@pytest.mark.parametrize('fail_test', [True, False])
def test_errors_in_cleanup(suite, suite_test, fail_test):
    cleanup1 = suite_test.add_cleanup()
    cleanup2 = suite_test.add_cleanup(extra_code=['None.a = 2'])

    if fail_test:
        suite_test.when_run.raise_exception()
    else:
        suite_test.expect_error()

    summary = suite.run()

    assert summary.events[cleanup1].timestamp > summary.events[cleanup2].timestamp

    [result] = summary.get_all_results_for_test(suite_test)

    assert len(result.get_errors()) == 2 if fail_test else 1
    cleanup_error = result.get_errors()[-1]
    assert 'AttributeError' in str(cleanup_error)
    assert 'NoneType' in str(cleanup_error)




import pytest
from slash.utils.function_marker import function_marker


def test_marked_func_identical_to_original(marker, func):
    orig_func = func
    new_func = marker(func)
    assert orig_func is new_func

def test_is_marked(marker, func):
    assert not marker.is_marked(func)
    marker(func)
    assert marker.is_marked(func)

@pytest.mark.parametrize('value', [True, False, 'value', 2.0])
def test_marker_value(func, value):

    marker = function_marker('bla')

    func = marker(value)(func)

    assert marker.is_marked(func)
    assert marker.get_value(func) == value

def test_marker_value_does_not_exist(func):
    with pytest.raises(LookupError):
        function_marker('bla').get_value(func)

    assert function_marker('bla').get_value(func, 1) == 1

def test_marker_on_methods(marker):

    class Obj(object):

        @marker
        def marked_func(self):
            pass

        def unmarked_func(self):
            pass

        @classmethod
        def unmarked_classmethod(self):
            pass

        @staticmethod
        def unmarked_staticmethod(self):
            pass

        @marker
        @classmethod
        def marked_classmethod(self):
            pass

        @marker
        @staticmethod
        def marked_staticmethod(self):
            pass


    for subject in (Obj, Obj()):
        for name in dir(subject):
            if name.startswith('_'):
                continue
            method = getattr(subject, name)
            assert 'marked' in name
            if 'unmarked' in name:
                assert not marker.is_marked(method)
            else:
                assert marker.is_marked(method)

@pytest.fixture
def marker():
    return function_marker('some_marker')

@pytest.fixture
def func():
    def returned():
        pass
    return returned



import pytest
import slash
from slash import context, Session
from .utils import run_tests_assert_success
from .utils.suite_writer import Suite


def test_current_test():
    suite = Suite()

    suite.add_test(type='method').prepend_line(
        'assert slash.test == self')
    suite.add_test(type='method').prepend_line(
        'assert slash.context.test == self')
    suite.add_test(type='method').prepend_line(
        'assert slash.context.test.__slash__.id == self.__slash__.id')

    suite.run()


def test_get_current_session():
    with Session() as s:
        assert context.session is s
        assert context.session is not slash.session
        assert s == slash.session


def test_test_methodname_has_no_dot(test_globals):
    assert not test_globals['test_methodname'].startswith('.')


def test_globals_dir():
    with Session():
        assert 'x' not in dir(slash.g)
        slash.g.x = 2
        assert 'x' in dir(slash.g)


@pytest.fixture
def test_globals(is_method):
    returned = {}

    def _distill():
        returned['test_methodname'] = context.test_methodname
    if is_method:
        class test_something(slash.Test):

            def test_something(self):
                _distill()
    else:
        def test_something():
            _distill()
    run_tests_assert_success(test_something)
    return returned


@pytest.fixture(params=[True, False])
def is_method(request):
    return request.param



import os

import dessert
from emport import import_file

class LocalConfig(object):

    def __init__(self):
        super(LocalConfig, self).__init__()
        self._slashconf_vars_cache = {}
        self._configs = []

    def push_path(self, path):
        self._configs.append(self._build_config(path))

    def pop_path(self):
        self._configs.pop(-1)

    def get_dict(self):
        return self._configs[-1]

    def _build_config(self, path):
        confstack = []
        for dir_path in self._traverse_upwards(path):
            slashconf_vars = self._slashconf_vars_cache.get(dir_path)
            if slashconf_vars is None:
                slashconf_path = os.path.join(dir_path, 'slashconf.py')
                if os.path.isfile(slashconf_path):
                    with dessert.rewrite_assertions_context():
                        slashconf_vars = self._slashconf_vars_cache[dir_path] = vars(import_file(slashconf_path))

            if slashconf_vars is not None:
                confstack.append(slashconf_vars)

        returned = {}
        # start loading from the parent so that vars are properly overriden
        for slashconf_vars in reversed(confstack):
            returned.update(slashconf_vars)
        return returned

    def _traverse_upwards(self, path):
        path = os.path.abspath(path)
        if not os.path.exists(path):
            raise RuntimeError("Path doesn't exist: {0}".format(path))

        if os.path.isfile(path):
            path = os.path.dirname(path)

        while True:
            yield path
            if path == os.path.abspath(os.path.sep):
                break
            new_path = os.path.dirname(path)
            assert new_path != path
            path = new_path



import itertools

_id_allocator = itertools.count()


class Element(object):

    def __init__(self, suite):
        super(Element, self).__init__()
        self.suite = suite
        self.id = '{0:05}'.format(next(_id_allocator))

    def __repr__(self):
        return '<{0} {1}>'.format(type(self).__name__, self.id)



import sys
from contextlib import contextmanager

import logbook

from . import hooks
from . import log
from ._compat import ExitStack
from .conf import config
from .ctx import context
from .exception_handling import handling_exceptions
from .exceptions import NoActiveSession, SkipTest, TestFailed
from .core.function_test import FunctionTest
from .core.metadata import ensure_test_metadata
from .utils.iteration import PeekableIterator
from .utils.interactive import notify_if_slow_context
from .core.error import Error


_logger = logbook.Logger(__name__)
log.set_log_color(_logger.name, logbook.NOTICE, 'teal')


def run_tests(iterable, stop_on_error=None):
    """
    Runs tests from an iterable using the current session
    """
    # pylint: disable=maybe-no-member
    if context.session is None or not context.session.started:
        raise NoActiveSession("A session is not currently started")

    if stop_on_error is None:
        stop_on_error = config.root.run.stop_on_error

    test_iterator = PeekableIterator(iterable)
    last_filename = None
    complete = False
    try:
        for test in test_iterator:
            _set_test_metadata(test)
            test_filename = test.__slash__.file_path
            if last_filename != test_filename:
                context.session.reporter.report_file_start(test_filename)
                last_filename = test_filename
            context.session.reporter.report_test_start(test)
            _logger.notice("{0}", test.__slash__.address)

            with _get_run_context_stack(test, test_iterator) as should_run:
                if should_run:
                    test.run()

            result = context.session.results[test]
            context.session.reporter.report_test_end(test, result)
            if not test_iterator.has_next() or ensure_test_metadata(test_iterator.peek()).file_path != last_filename:
                context.session.reporter.report_file_end(last_filename)
            if result.has_fatal_exception():
                _logger.debug("Stopping on fatal exception")
                break
            if not result.is_success() and not result.is_skip() and stop_on_error:
                _logger.debug("Stopping (run.stop_on_error==True)")
                break
        else:
            complete = True
    finally:
        context.session.scope_manager.flush_remaining_scopes(
            in_failure=not complete, in_interruption=context.session.results.is_interrupted())

    _mark_unrun_tests(test_iterator)
    if complete:
        context.session.mark_complete()
    elif last_filename is not None:
        context.session.reporter.report_file_end(last_filename)
    _logger.debug('Session finished. is_success={0} has_skips={1}',
                  context.session.results.is_success(allow_skips=True), bool(context.session.results.get_num_skipped()))


def _set_test_metadata(test):
    ensure_test_metadata(test)
    assert test.__slash__.test_index0 is None
    test.__slash__.test_index0 = next(context.session.test_index_counter)


def _mark_unrun_tests(test_iterator):
    remaining = list(test_iterator)
    for test in remaining:
        with _get_test_context(test, logging=False):
            pass


@contextmanager
def _get_run_context_stack(test, test_iterator):
    yielded = False
    # run_state is needed to support Python 2.6, where exc_info() disappears in some cases when using finally
    run_state = {'exc_info': (None, None, None)}
    with ExitStack() as stack:
        stack.enter_context(_get_test_context(test))

        if not _check_test_requirements(test):
            yield False
            return

        stack.enter_context(handling_exceptions())
        stack.enter_context(_get_test_hooks_context())
        stack.enter_context(_update_result_context())
        stack.enter_context(_get_test_scope_context(test, test_iterator, run_state))
        stack.enter_context(handling_exceptions())
        yielded = True
        try:
            yield True
        except:
            run_state['exc_info'] = sys.exc_info()
            raise

    # if some of the context entries throw SkipTest, the yield result above will not be reached.
    # we have to make sure that yield happens or else the context manager will raise on __exit__...
    if not yielded:
        yield False


def _check_test_requirements(test):
    unmet_reqs = test.get_unmet_requirements()
    if unmet_reqs:
        context.result.add_skip('Unmet requirements: {0}'.format(unmet_reqs))
        return False
    return True


@contextmanager
def _get_test_scope_context(test, test_iterator, run_state):
    context.session.scope_manager.begin_test(test)
    try:
        yield
    finally:
        context.session.scope_manager.end_test(test, next_test=test_iterator.peek_or_none(), exc_info=run_state['exc_info'])


@contextmanager
def _get_test_context(test, logging=True):
    ensure_test_metadata(test)

    assert test.__slash__.id is None
    test.__slash__.id = context.session.id_space.allocate()
    with _set_current_test_context(test):
        result = context.session.results.create_result(test)
        prev_result = context.result
        context.result = result
        try:
            with (context.session.logging.get_test_logging_context() if logging else ExitStack()):
                _logger.debug("Started test: {0}", test)
                yield
        finally:
            context.result = prev_result


@contextmanager
def _get_test_hooks_context():
    hooks.test_start()  # pylint: disable=no-member
    try:
        yield
    except SkipTest as skip_exception:
        hooks.test_skip(reason=skip_exception.reason)  # pylint: disable=no-member
    except TestFailed:
        hooks.test_failure()  # pylint: disable=no-member
    except KeyboardInterrupt:
        with notify_if_slow_context(message="Cleaning up due to interrupt. Please wait..."):
            hooks.test_interrupt()  # pylint: disable=no-member
        raise
    except:
        hooks.test_error()  # pylint: disable=no-member
    else:
        res = context.session.results.get_result(context.test)
        if res.is_success_finished():
            hooks.test_success()  # pylint: disable=no-member
        elif res.is_just_failure():
            hooks.test_failure()  # pylint: disable=no-member
        else:
            hooks.test_error()  # pylint: disable=no-member
    finally:
        hooks.test_end()  # pylint: disable=no-member


@contextmanager
def _set_current_test_context(test):
    prev_test = context.test
    prev_test_id = context.test_id
    context.test = test
    context.test_id = test.__slash__.id
    if isinstance(test, FunctionTest):
        context.test_classname = None
        context.test_methodname = test.__slash__.factory_name
    else:
        context.test_classname = test.__slash__.factory_name
        # this includes a dot (.), so it has to be truncated
        context.test_methodname = test.__slash__.address_in_factory[1:]
    try:
        yield
    finally:
        context.test = prev_test
        context.test_id = prev_test_id


@contextmanager
def _update_result_context():
    result = context.result
    assert result
    result.mark_started()
    try:
        with handling_exceptions():
            yield result
    except SkipTest as e:
        result.add_skip(e.reason)
        raise
    except KeyboardInterrupt:
        result.mark_interrupted()
        raise
    except Exception as e:
        _logger.debug("Exception escaped test:\n{0}", Error.capture_exception().get_detailed_str())
        raise
    finally:
        result.mark_finished()



import itertools
from contextlib import contextmanager
from uuid import uuid4

from .code_element import CodeElement
from .parameter import Parameter


class Function(CodeElement):

    def __init__(self, suite):
        super(Function, self).__init__(suite)
        self._decorators = []
        self._parameters = []
        self._additional_parameter_string = ""
        self._fixtures = []
        self._deferred_events = []

    def add_parameter_string(self, s):
        self._additional_parameter_string += s

    def add_decorator(self, decorator_string):
        self._decorators.append(decorator_string)

    def get_fixtures(self):
        return self._fixtures

    def get_parameters(self):
        return self._parameters

    def add_parameter(self, *args, **kwargs):
        returned = Parameter(self.suite, *args, **kwargs)
        self._parameters.append(returned)
        return returned

    def depend_on_fixture(self, f):
        self._fixtures.append(f)
        return f

    def _write_event(self, code_formatter, eventcode):
        if self.suite.debug_info:
            code_formatter.writeln(
                '__ut__.events.add({0!r}, {1!r})'.format(
                    eventcode, self.id))

    def add_deferred_event(self, decorator=None, name='deferred', extra_code=(), adder=None):
        event = '{0}_{1}'.format(name, uuid4())
        self._deferred_events.append({
            'decorator': decorator, 'event': event, 'extra_code': extra_code, 'adder': adder})
        return event

    @contextmanager
    def _body_context(self, code_formatter):
        self._write_decorators(code_formatter)
        code_formatter.writeln('def {0}({1}):'.format(
            self._get_function_name(),
            self._get_parameter_string()))

        with code_formatter.indented():
            if not self.suite.debug_info:
                code_formatter.writeln('pass')
            self._write_parameter_values(code_formatter)
            self._write_deferred_events(code_formatter)
            self._write_prologue(code_formatter)
            yield
            self._write_epilogue(code_formatter)
            self._write_return(code_formatter)
        code_formatter.writeln()

    def _get_parameter_string(self):
        returned = ', '.join(self._get_argument_names())
        if returned and self._additional_parameter_string:
            returned += ', '
        returned += self._additional_parameter_string
        return returned

    def _write_prologue(self, code_formatter):
        pass

    def _write_epilogue(self, code_formatter):
        pass

    def _write_deferred_events(self, code_formatter):
        if not self.suite.debug_info:
            return
        for index, deferred in enumerate(self._deferred_events, 1):
            deferred_func_name = '_deferred{0}'.format(index)
            adder = deferred['adder']
            if adder is None:
                code_formatter.writeln('@{0[decorator]}'.format(deferred))
            code_formatter.writeln('def {0}():'.format(deferred_func_name))
            with code_formatter.indented():
                code_formatter.writeln('__ut__.events.add({0[event]!r})'.format(deferred))
                for line in deferred['extra_code']:
                    code_formatter.writeln(line)
            if adder is not None:
                code_formatter.write(adder.format(deferred_func_name))
            code_formatter.writeln()

    def _write_return(self, code_formatter):
        pass

    def _write_decorators(self, code_formatter):
        for d in self._decorators:
            code_formatter.write('@')
            code_formatter.writeln(d)
        for p in self._parameters:
            p.write_decorator(code_formatter)

    def _write_parameter_values(self, code_formatter):
        if not self.suite.debug_info:
            return
        for p in self._parameters:
            code_formatter.writeln('__ut__.notify_parameter_value({0!r}, {1})'.format(
                p.id, p.name))

    def _get_function_name(self):
        raise NotImplementedError()  # pragma: no cover

    @property
    def name(self):
        return self._get_function_name()

    def _get_argument_names(self):
        return (p.name for p in itertools.chain(self._parameters, self._fixtures))



from .exceptions import TestFailed
from .utils import operator_information
from contextlib import contextmanager
import operator
import sys

sys.modules["slash.should"] = sys.modules[__name__]


def _binary_assertion(name, operator_func):
    op = operator_information.get_operator_by_func(operator_func)

    def _assertion(a, b, msg=None):
        if not op(a, b):
            msg = _get_message(msg, operator_information.get_operator_by_func(
                op.inverse_func).to_expression(a, b))
            raise TestFailed(msg)
    _assertion.__name__ = name
    _assertion.__doc__ = "Asserts **{0}**".format(
        op.to_expression("ARG1", "ARG2"))
    return _assertion


def _unary_assertion(name, operator_func):
    op = operator_information.get_operator_by_func(operator_func)

    def _assertion(a, msg=None):
        if not op(a):
            msg = _get_message(msg, operator_information.get_operator_by_func(
                op.inverse_func).to_expression(a))
            raise TestFailed(msg)
    _assertion.__name__ = name
    _assertion.__doc__ = "Asserts **{0}**".format(op.to_expression("ARG"))
    return _assertion


def _get_message(msg, description):
    if msg is None:
        return description
    return "{0} ({1})".format(msg, description)

equal = _binary_assertion("equal", operator.eq)
assert_equal = assert_equals = equal = equal

not_equal = _binary_assertion("not_equal", operator.ne)
assert_not_equal = assert_not_equals = not_equals = not_equal

be_a = _binary_assertion("be_a", operator_information.safe_isinstance)
assert_isinstance = be_a

not_be_a = _binary_assertion(
    "not_be_a", operator_information.safe_not_isinstance)
assert_not_isinstance = not_be_a

be_none = _unary_assertion("be_none", operator_information.is_none)
assert_is_none = be_none

not_be_none = _unary_assertion("not_be_none", operator_information.is_not_none)
assert_is_not_none = not_be_none

be = _binary_assertion("be", operator.is_)
assert_is = be

not_be = _binary_assertion("not_be", operator.is_not)
assert_is_not = not_be

be_true = _unary_assertion("be_true", operator.truth)
assert_true = be_true

be_false = _unary_assertion("be_false", operator.not_)
assert_false = be_false

be_empty = _unary_assertion("be_empty", operator_information.is_empty)
assert_empty = assert_is_empty = be_empty

not_be_empty = _unary_assertion(
    "not_be_empty", operator_information.is_not_empty)
assert_not_empty = assert_is_not_empty = not_be_empty

contain = _binary_assertion("contain", operator.contains)
assert_contains = contains = contain

not_contain = _binary_assertion(
    "not_contain", operator_information.not_contains)
assert_not_contains = assert_not_contain = not_contains = not_contain


def be_in(a, b, msg=None):
    """
    Asserts **ARG1 in ARG2**
    """
    return contain(b, a, msg)
assert_in = be_in


def not_be_in(a, b, msg=None):
    """
    Asserts **ARG1 not in ARG2**
    """
    return not_contain(b, a, msg)
assert_not_in = not_be_in


@contextmanager
def raise_exception(exception_class, msg=None):
    """
    Ensures a subclass of **ARG1** leaves the wrapped context:

    >>> with raise_exception(AttributeError):
    ...     raise AttributeError()
    """
    caught = _CaughtException()
    try:
        yield caught
    except exception_class as e:
        caught.exception = e
    else:
        expected_classes = exception_class
        if msg is None:
            if not isinstance(expected_classes, tuple):
                expected_classes = (expected_classes,)
            msg = "{0} not raised".format("/".join(e.__name__ for e in expected_classes))
        raise TestFailed(msg)


def assert_raises(exception_class, msg=None):
    return raise_exception(exception_class, msg=msg)
assert_raises.__doc__ = raise_exception.__doc__.replace(
    "raise_exception", "assert_raises")


def assert_almost_equal(a, b, delta=0.00000001):
    """Asserts that abs(a - b) <= delta
    """
    assert abs(a - b) <= delta


class _CaughtException(object):
    exception = None

# def assertDictEqual(self, d1, d2, msg=None):
# def assertDictContainsSubset(self, expected, actual, msg=None):
# def assertItemsEqual(self, expected_seq, actual_seq, msg=None):
# def assertMultiLineEqual(self, first, second, msg=None):
# def assertLess(self, a, b, msg=None):
# def assertLessEqual(self, a, b, msg=None):
# def assertGreater(self, a, b, msg=None):
# def assertGreaterEqual(self, a, b, msg=None):
# def assertRaisesRegexp(self, expected_exception, expected_regexp,
# def assertRegexpMatches(self, text, expected_regexp, msg=None):
# def assertNotRegexpMatches(self, text, unexpected_regexp, msg=None):
# def assertSequenceEqual(self, seq1, seq2, msg=None, seq_type=None):
# def assertAlmostEqual(self, first, second, places=None, msg=None, delta=None):
# def assertNotAlmostEqual(self, first, second, places=None, msg=None, delta=None):
#



from .utils import (
    TestCase,
    CustomException,
    run_tests_in_session,
    make_runnable_tests,
    )
import itertools
import slash
from slash.loader import Loader

class TestTest(TestCase):
    """
    Test the :class:`Test` class, which is the quickest way to create test classes in Slash
    """
    def test_test_class(self):
        events = []
        class Test(slash.Test):
            def before(self):
                events.append("before")
            def after(self):
                events.append("after")
            def test_1(self):
                events.append("test_1")
            def test_2(self):
                events.append("test_2")
        with slash.Session():
            tests = make_runnable_tests(Test)
        for test in tests:
            self.assertIsInstance(test, Test)
        self.assertEquals(len(tests), 2)
        tests.sort(key=lambda test: test._test_method_name)
        for test in tests:
            test.run()
        self.assertEquals(events, ["before", "test_1", "after", "before", "test_2", "after"])

    def test_before_failures(self):
        "Check that exceptions during before() prevent after() from happening"
        events = []
        class Test(slash.Test):
            def before(self):
                raise CustomException()
            def test(self):
                events.append("test")
            def after(self):
                events.append("after")
        with slash.Session():
            [test] = make_runnable_tests(Test)
        with self.assertRaises(CustomException):
            test.run()
        self.assertEquals(events, [])

    def test_after_failures(self):
        class Test(slash.Test):
            def test(self):
                assert False, "msg1"

            def after(self):
                assert False, "msg2"

        session = run_tests_in_session(Test)
        self.assertFalse(session.results.is_success())
        [result] = session.results.iter_test_results()
        self.assertEquals(len(result.get_failures()), 2)

    def test_after_gets_called(self):
        "If before() is successful, after() always gets called"
        events = []
        class Test(slash.Test):
            def before(self):
                events.append("before")
            def test_1(self):
                events.append("test")
                raise CustomException(1)
            def after(self):
                events.append("after")
        with slash.Session():
            [test] = make_runnable_tests(Test)
        with self.assertRaises(CustomException):
            test.run()
        self.assertEquals(events, ["before", "test", "after"])

class AbstractTestTest(TestCase):
    def test_abstract_tests(self):
        @slash.abstract_test_class
        class Abstract(slash.Test):
            def test1(self):
                pass
            def test2(self):
                pass
            def test3(self):
                pass
        with slash.Session():
            self.assertEquals(list(make_runnable_tests(Abstract)), [])
        class Derived(Abstract):
            pass
        with slash.Session():
            self.assertEquals(len(list(make_runnable_tests(Derived))), 3)

class TestParametersTest(TestCase):
    def test_parameters(self):
        variations = []
        a_values = [1, 2]
        b_values = [3, 4]
        c_values = [5, 6]
        d_values = [7, 8]
        class Parameterized(slash.Test):
            @slash.parameters.iterate(a=a_values)
            def before(self, a):
                variations.append([a])
            @slash.parameters.iterate(b=b_values, c=c_values)
            def test(self, b, c):
                variations[-1].extend([b, c])
            @slash.parameters.iterate(d=d_values)
            def after(self, d):
                variations[-1].append(d)
        with slash.Session():
            for test in make_runnable_tests(Parameterized):
                test.run()
        self.assertEquals(
            set(tuple(x) for x in variations),
            set(itertools.product(
                a_values,
                b_values,
                c_values,
                d_values
            )))



from sentinels import NOTHING

from .._compat import get_underlying_classmethod_function


def function_marker(name):
    return Marker(name)

class Marker(object):

    def __init__(self, name, value=True):
        super(Marker, self).__init__()
        self._name = name
        self._value = value
        self._mark = '__marked_{0}__'.format(self._name)

    def __call__(self, func):
        if not hasattr(func, '__call__') and not isinstance(func, (classmethod, staticmethod)):
            return Marker(self._name, func)

        setattr(self._normalize(func), self._mark, self._value)
        return func

    def is_marked(self, func):
        return hasattr(self._normalize(func), self._mark)

    def get_value(self, func, default=NOTHING):
        returned = getattr(self._normalize(func), self._mark, default)
        if returned is NOTHING:
            raise LookupError()
        return returned

    @staticmethod
    def _normalize(func):
        if isinstance(func, (classmethod, staticmethod)):
            return get_underlying_classmethod_function(func)
        return func



import pytest
import slash

from .utils import TestCase, make_runnable_tests


@pytest.mark.parametrize("args", [(), ("message",)])
def test_skip_test(args):
    "Make sure the skip_test function raises a SkipTest exception"
    with pytest.raises(slash.exceptions.SkipTest) as caught:
        slash.skip_test(*args)
    if args:
        assert caught.value.reason == args[0]


def test_skip_with_before_after_test(checkpoint):
    "Make sure that after() is called for Test even if we skip"
    class MyTest(slash.Test):

        def test(self):
            slash.skip_test("!")

        def after(self):
            checkpoint()

    _assert_skips(MyTest, reason="!")
    assert checkpoint.called


def test_method_without_reason():
    class Test(slash.Test):

        @slash.skipped
        def test(self):
            pass

    _assert_skips(Test)


def test_method_with_reason():

    class Test(slash.Test):

        @slash.skipped("reason")
        def test(self):
            pass
    _assert_skips(Test, "reason")


def test_class_decorator(suite):

    cls = suite.classes[1]

    cls.add_decorator('slash.skipped("reason")')

    for test in cls.tests:
        test.expect_skip()

    results = suite.run()

    for test in cls.tests:
        result = results[test]
        assert 'reason' in result.get_skips()


def _assert_skips(thing, reason=None):
    with slash.Session():
        if isinstance(thing, type) and issubclass(thing, slash.Test):
            [thing] = make_runnable_tests(thing)
            thing = thing.run

    with pytest.raises(slash.exceptions.SkipTest) as caught:
        thing()
    assert caught.value.reason == reason



from .utils import TestCase
import os
import sys
from tempfile import mktemp
import slash
import slash.site
from slash._compat import cStringIO
from slash.frontend import slash_run
import requests
import pkg_resources

class SlashRunSiteCustomizationTest(TestCase):
    "Make sure ``slash run`` calls site.load()"
    def setUp(self):
        super(SlashRunSiteCustomizationTest, self).setUp()
        self.forge.replace(slash.site, "load")
        self.forge.replace_with(sys, "stderr", cStringIO())
    def test_slash_run_calls_site_load(self):
        slash.site.load()
        self.forge.replay()
        with self.assertRaises(SystemExit):
            slash_run.slash_run([])

_customization_index = 0
_loaded_customizations = []

def _apply_customization(index=0):
    _loaded_customizations.append(index)

class CustomizationTest(TestCase):

    def setUp(self):
        super(CustomizationTest, self).setUp()
        global _loaded_customizations
        global _customization_index
        _customization_index = 0
        _loaded_customizations = []

    def get_customization_source(self):
        global _customization_index
        returned = "import {0}; {0}._apply_customization({1})".format(__name__, _customization_index)
        _customization_index += 1
        return returned

    def test_customize_via_local_and_global_slashrc(self):
        self._test_customize_via_local_slashrc(also_use_global=True)

    def test_customize_via_local_slashrc(self):
        self._test_customize_via_local_slashrc(also_use_global=False)

    def _test_customize_via_local_slashrc(self, also_use_global):
        if also_use_global:
            global_slashrc_path = os.path.join(self.get_new_path(), "slashrc")
            self.override_config("run.user_customization_file_path", global_slashrc_path)
            with open(global_slashrc_path, "w") as f:
                f.write(self.get_customization_source())

        self.addCleanup(os.chdir, os.path.abspath("."))
        os.chdir(self.get_new_path())
        with open(".slashrc", "w") as f:
            f.write(self.get_customization_source())
        self.assert_customization_loaded()

    def test_customize_via_env_var(self):
        os.environ["SLASH_SETTINGS"] = custom_filename = mktemp()
        self.addCleanup(os.environ.pop, "SLASH_SETTINGS")
        with open(custom_filename, "w") as f:
            f.write(self.get_customization_source())
        self.assert_customization_loaded()

    def test_customize_via_url(self):
        url = "http://nonexistent.com/some/path/to/custom/file.py"
        self.forge.replace(requests, "get")
        fake_response = self.forge.create_mock(requests.Response)
        fake_response.raise_for_status().whenever()
        fake_response.content = self.get_customization_source()
        requests.get(url).and_return(fake_response)
        os.environ["SLASH_SETTINGS"] = url
        self.addCleanup(os.environ.pop, "SLASH_SETTINGS")
        self.forge.replay()
        self.assert_customization_loaded()

    def test_customize_via_pkgutil_entry_point(self):
        self.forge.replace(pkg_resources, "iter_entry_points")
        entry_point = self.forge.create_wildcard_mock()
        pkg_resources.iter_entry_points("slash.site.customize").and_return(iter([entry_point]))
        unused = self.get_customization_source() # expect a single customization
        entry_point.load().and_return(_apply_customization)
        self.forge.replay()
        self.assert_customization_loaded()

    def assert_customization_loaded(self):
        global _loaded_customizations
        global _customization_index
        slash.site.load()
        self.assertEquals(_loaded_customizations, list(range(_customization_index)))



from .__version__ import __version__
from .cleanups import add_cleanup, add_critical_cleanup, add_success_only_cleanup
from .conf import config
from .ctx import context
from .ctx import g, session, test
from .core.session import Session
# assertions
from . import assertions
should = assertions
from .assertions import (
    assert_almost_equal,
    assert_contains,
    assert_equal,
    assert_equals,
    assert_false,
    assert_in,
    assert_is,
    assert_is_none,
    assert_empty,
    assert_not_empty,
    assert_is_not,
    assert_is_not_none,
    assert_isinstance,
    assert_not_contain,
    assert_not_contains,
    assert_not_equal,
    assert_not_equals,
    assert_not_in,
    assert_not_isinstance,
    assert_raises,
    assert_true,
    )
from .core.test import Test
from .core.test import abstract_test_class
from .core.fixtures import parametrize, parameters
from .core.fixtures.utils import fixture, nofixtures
from .core.requirements import requires
from .utils import skip_test, skipped, add_error, add_failure, set_test_detail, repeat
from .utils.interactive import start_interactive_shell
from .app import get_application_context
from .runner import run_tests
import logbook
logger = logbook.Logger(__name__)




import sys

from . import context
from .conf import config
from .utils.path import ensure_containing_directory
from .warnings import WarnHandler
from contextlib import contextmanager
import logbook  # pylint: disable=F0401
import logbook.more
from ._compat import ExitStack
import os
import numbers

_logger = logbook.Logger(__name__)

_custom_colors = {}


class _NormalizedObject(object):
    def __init__(self, obj):
        self._obj = obj

    def __getattr__(self, name):
        obj = getattr(self._obj, name)
        return obj if isinstance(obj, numbers.Number) else _NormalizedObject(obj)

    @staticmethod
    def _escape(s):
        return s.replace('\\', '_').replace('/', '_')

    def __str__(self):
        return self._escape(str(self._obj))

    def __repr__(self):
        return self._escape(repr(self._obj))


def set_log_color(logger_name, level, color):
    """Sets the color displayed in the console, according to the logger name and level
    """
    _custom_colors[logger_name, level] = color

class ColorizedHandlerMixin(logbook.more.ColorizingStreamHandlerMixin):

    def get_color(self, record):
        returned = _custom_colors.get((record.channel, record.level))
        if returned is not None:
            return returned

        if record.level >= logbook.ERROR:
            return 'red'
        elif record.level >= logbook.WARNING:
            return 'yellow'
        elif record.level >= logbook.NOTICE:
            return 'white'
        return None # default

class ColorizedFileHandler(ColorizedHandlerMixin, logbook.FileHandler):

    def should_colorize(self, record):
        return True


class ConsoleHandler(ColorizedHandlerMixin, logbook.StreamHandler):

    MAX_LINE_LENGTH = 160

    default_format_string = '[{record.time:%Y-%m-%d %H:%M:%S}] {record.message}'

    def __init__(self, **kw):
        stream = kw.pop('stream', sys.stderr)
        logbook.StreamHandler.__init__(self, stream=stream, **kw)
        self._truncate_lines = config.root.log.truncate_console_lines
        self._truncate_errors = config.root.log.truncate_console_errors


    def format(self, record):
        orig_message = record.message
        should_truncate = self._truncate_errors or record.level < logbook.ERROR
        if self._truncate_lines and len(orig_message) > self.MAX_LINE_LENGTH and should_truncate:
            record.message = "\n".join(self._truncate(line) for line in orig_message.splitlines())
        try:
            returned = super(ConsoleHandler, self).format(record)
        finally:
            # we back up the original message to avoid propagating truncated lines into the file log
            record.message = orig_message
        return returned

    def _truncate(self, line):
        if len(line) > self.MAX_LINE_LENGTH:
            line = line[:self.MAX_LINE_LENGTH - 3] + "..."
        return line

    def emit(self, record):
        context.session.reporter.notify_before_console_output()
        returned = super(ConsoleHandler, self).emit(record)
        context.session.reporter.notify_after_console_output()
        return returned

class SessionLogging(object):
    """
    A context creator for logging within a session and its tests
    """
    def __init__(self, session, console_stream=None):
        super(SessionLogging, self).__init__()
        if console_stream is None:
            console_stream = sys.stderr
        self.session = session
        self.warnings_handler = WarnHandler(session.warnings)
        self.console_handler = ConsoleHandler(bubble=True, level=config.root.log.console_level, stream=console_stream)
        #: contains the path for the session logs
        self.session_log_path = None
        self.session_log_handler = None
        #: contains the path for the current test logs
        self.test_log_path = None
        self._set_formatting(self.console_handler, config.root.log.console_format or config.root.log.format)

    @contextmanager
    def get_test_logging_context(self):
        with self._get_file_logging_context(config.root.log.subpath, config.root.log.last_test_symlink) as (_, path):
            self.test_log_path = path
            context.result.set_log_path(path)
            try:
                yield path
            finally:
                self._create_last_failed_symlink_if_needed()

    def _create_last_failed_symlink_if_needed(self):
        result = context.result
        assert result
        if result.is_error() or result.is_failure():
            self._try_create_symlink(result.get_log_path(), config.root.log.last_failed_symlink)

    @contextmanager
    def get_session_logging_context(self):
        assert self.session_log_handler is None
        with self._get_file_logging_context(
            config.root.log.session_subpath, config.root.log.last_session_symlink) as (handler, path):
            self.session_log_handler = handler
            self.session_log_path = path
            self.session.results.global_result.set_log_path(path)
            if config.root.log.last_session_dir_symlink is not None and self.session_log_path is not None:
                self._try_create_symlink(os.path.dirname(self.session_log_path), config.root.log.last_session_dir_symlink)
            yield path

    @contextmanager
    def _get_file_logging_context(self, filename_template, symlink):
        with ExitStack() as stack:
            handler, path = self._get_file_log_handler(filename_template, symlink)
            stack.enter_context(handler.applicationbound())
            stack.enter_context(self.console_handler.applicationbound())
            stack.enter_context(self.warnings_handler.applicationbound())
            stack.enter_context(self._get_silenced_logs_context())
            if config.root.log.unittest_mode:
                stack.enter_context(logbook.StreamHandler(sys.stderr, bubble=True))
            for extra_handler in _extra_handlers:
                stack.enter_context(extra_handler.applicationbound())
            if config.root.log.unified_session_log and self.session_log_handler is not None:
                stack.enter_context(self.session_log_handler)

            yield handler, path

    def _get_silenced_logs_context(self):
        if not config.root.log.silence_loggers:
            return ExitStack()
        return SilencedLoggersHandler(config.root.log.silence_loggers).applicationbound()

    def _get_file_log_handler(self, subpath, symlink):
        root_path = config.root.log.root
        if root_path is None:
            log_path = None
            handler = logbook.NullHandler(bubble=False)
        else:
            log_path = self._normalize_path(os.path.join(root_path, subpath.format(context=_NormalizedObject(context))))
            ensure_containing_directory(log_path)
            handler = self._get_file_handler_class()(log_path, bubble=False)
            self._try_create_symlink(log_path, symlink)
            self._set_formatting(handler, config.root.log.format)
        return handler, log_path

    def _get_file_handler_class(self):
        if config.root.log.colorize:
            return ColorizedFileHandler
        return logbook.FileHandler

    def _normalize_path(self, p):
        return os.path.expanduser(p)

    def _try_create_symlink(self, path, symlink):
        if symlink is None or config.root.log.root is None:
            return

        symlink = self._normalize_path(symlink)

        if not os.path.isabs(symlink):
            symlink = os.path.join(self._normalize_path(config.root.log.root), symlink)

        try:
            ensure_containing_directory(symlink)

            if os.path.exists(symlink) or os.path.islink(symlink):
                os.unlink(symlink)
            os.symlink(path, symlink)

        except Exception:  # pylint: disable=broad-except
            _logger.debug("Failed to create symlink {0} --> {1}", path, symlink, exc_info=True)

    def _set_formatting(self, handler, fmt):
        if config.root.log.localtime:
            logbook.set_datetime_format("local")
        if fmt is not None:
            handler.format_string = fmt

class SilencedLoggersHandler(logbook.Handler):
    def __init__(self, silence_logger_names):
        super(SilencedLoggersHandler, self).__init__(bubble=False)
        self._silenced_names = set(silence_logger_names)
    def should_handle(self, record):
        return record.channel in self._silenced_names

def add_log_handler(handler):
    """
    Adds a log handler to be entered for sessions and for tests
    """
    _extra_handlers.append(handler)

def remove_all_extra_handlers():
    del _extra_handlers[:]

_extra_handlers = []

class VERBOSITIES(object):

    DEBUG = logbook.DEBUG
    INFO = logbook.INFO
    NOTICE = logbook.NOTICE
    WARNING = logbook.WARNING
    ERROR = logbook.ERROR
    CRITICAL = logbook.CRITICAL



import pytest


@pytest.mark.parametrize('scope', ['test', 'session'])
@pytest.mark.parametrize('use_skip', [True, False])
def test_exceptions_in_fixture(suite, suite_test, scope, use_skip):
    second_test = suite_test.file.add_function_test()

    fixture = suite_test.file.add_fixture(scope=scope)
    suite_test.depend_on_fixture(fixture)
    second_test.depend_on_fixture(fixture)

    if use_skip:
        fixture.append_line('slash.skip_test()')
    else:
        fixture.append_line('assert False')

    for test in suite_test, second_test:
        if use_skip:
            test.expect_skip()
        else:
            test.expect_failure()

    suite.run()



import itertools

import logbook

from slash._compat import iteritems, itervalues, xrange

_logger = logbook.Logger(__name__)


def validate_run(suite, run_result, expect_interruption):
    if expect_interruption or not run_result.session.results.is_success(allow_skips=True):
        assert run_result.exit_code != 0, 'slash run unexpectedly returned 0'
    else:
        assert run_result.exit_code == 0

    for test, results in iteritems(_group_results_by_test_id(suite, run_result)):
        _validate_single_test(test, results)


def _validate_single_test(test, results):

    for param_values in _iter_param_value_sets(test):

        for repetition in xrange(test.get_num_expected_repetitions()):

            for index, result in enumerate(results):

                if _result_matches(test, result, param_values):

                    results.pop(index)

                    _validate_single_test_result(test, result)

                    break
            else:
                assert False, 'Could not find parameter set {0}'.format(
                    param_values)

    assert not results, 'Unmatched results exist'


def _iter_param_value_sets(test):
    params = _find_all_parameters(test)
    param_ids = [p.id for p in params]
    for combination in itertools.product(*(param.values for param in params)):
        yield dict(zip(param_ids, combination))


def _find_all_parameters(func):
    returned = {}
    stack = [func]
    while stack:
        f = stack.pop()
        stack.extend(f.get_fixtures())
        for param in f.get_parameters():
            if param.id not in returned:
                returned[param.id] = param
    return list(itervalues(returned))


def _result_matches(test, result, param_values):
    return result.data.get('param_values', {}) == param_values


def _validate_single_test_result(test, result):
    expected = test.get_expected_result()
    if expected == 'ERROR':
        assert result.is_error(), 'Test did not issue error as expected'
    elif expected == 'FAIL':
        assert result.is_failure(), 'Test did not fail as expected'
    elif expected == 'SUCCESS':
        assert result.is_success(), 'Test {0} unexpectedly unsuccessful:\n{1}'.format(
            test.id, list(itertools.chain(result.get_errors(), result.get_failures())))
    elif expected == 'INTERRUPT':
        assert result.is_interrupted(), 'Test did not get interrupted as expected'
    elif expected == 'SKIP':
        assert result.is_skip()
    elif expected == 'NOT_RUN':
        assert result.is_not_run()
    else:
        raise NotImplementedError(
            'Unknown expected result: {0!r}'.format(expected))  # pragma: no cover


def _group_results_by_test_id(suite, run_result):
    tests_by_id = dict((t.id, t) for t in suite)
    unseen = tests_by_id.copy()

    groups = {}

    for result in run_result.session.results:
        if 'Interactive' ==  result.test_metadata.address:
            continue
        test_id = get_test_id_from_test_address(result.test_metadata.address)
        assert tests_by_id[test_id].is_selected()
        groups.setdefault(tests_by_id[test_id], []).append(result)
        unseen.pop(test_id, None)

    for test_id, test in list(iteritems(unseen)):
        if not test.is_selected():
            unseen.pop(test_id, None)

    assert not unseen, 'Expected results not found ({0})'.format(unseen)

    return groups


def get_test_id_from_test_address(addr):
    return addr.rsplit('.', 1)[-1].split('_', 1)[1].split('(')[0]



from uuid import uuid1

import pytest
import slash

from .utils import run_tests_in_session


def test_fixture_cleanup(suite, suite_test):
    fixture = suite.slashconf.add_fixture()
    for i in range(3):
        fixture.add_cleanup()
    suite_test.depend_on_fixture(fixture)
    suite.run()


@pytest.mark.parametrize('scope', ['session', 'module'])
def test_fixture_autouse_with_scoping(suite, suite_test, scope):

    fixture = suite_test.file.add_fixture(autouse=True, scope=scope)
    c = fixture.add_cleanup()

    summary = suite.run()



import pytest

from slash.utils.python import call_all_raise_first


def test_call_all_raise_first(funcs):
    exc_type = funcs[2].raise_when_called()

    with pytest.raises(exc_type):
        call_all_raise_first(funcs)

    for index, func in enumerate(funcs):
        assert func.called_count == 1


@pytest.fixture
def funcs():

    class Func(object):

        called_count = 0
        exc_type = None

        def __call__(self):
            self.called_count += 1
            if self.exc_type is not None:
                raise self.exc_type()

        def raise_when_called(self):
            class CustomException(Exception):
                pass

            self.exc_type = CustomException
            return self.exc_type

    return [Func() for i in range(10)]



import os

import pytest
from slash import Session
from slash.exceptions import CannotLoadTests
from slash.loader import Loader

from .utils.suite_writer import Suite

@pytest.mark.parametrize('specific_method', [True, False])
@pytest.mark.parametrize('with_parameters', [True, False])
def test_iter_specific_factory(suite, suite_test, specific_method, with_parameters):

    if suite_test.cls is not None and specific_method:
        suite_test.cls.add_method_test()

    if with_parameters:
        suite_test.add_parameter()

    for test in suite:
        if suite_test.cls is None and test is not suite_test:
            # we are selecting a specific function, and that's not it:
            test.expect_deselect()
        elif suite_test.cls is not None and test.cls is not suite_test.cls:
            test.expect_deselect()
        elif specific_method and suite_test.cls is test.cls and suite_test is not test:
            test.expect_deselect()

    path = suite.commit()
    if suite_test.cls:
        assert suite_test.cls.tests
        factory_name = suite_test.cls.name
    else:
        factory_name = suite_test.name

    pattern = '{0}:{1}'.format(os.path.join(path, suite_test.file.get_relative_path()), factory_name)
    if suite_test.cls is not None and specific_method:
        assert len(suite_test.cls.tests) > 1
        pattern += '.{0}'.format(suite_test.name)
    suite.run(args=[pattern])


def test_import_error_registers_as_session_error(active_slash_session, test_loader):
    with pytest.raises(CannotLoadTests):
        test_loader.get_runnables(["/non/existent/path"])
    errors = active_slash_session.results.global_result.get_errors()
    assert len(errors) == 1
    [error] = errors


def test_import_errors_with_session():

    suite = Suite()

    for i in range(20):
        suite.add_test()

    problematic = suite.files[1]
    problematic.prepend_line('from nonexistent import nonexistent')

    for test in suite:
        test.expect_deselect()

    summary = suite.run()

    assert 0 != summary.exit_code

    errs = summary.session.results.global_result.get_errors()
    for err in errs:
        assert 'No module named nonexistent' in err.message or "No module named 'nonexistent'" in err.message


    return suite



import functools
import os

import logbook

import gossip
import pytest
import slash

from .utils import run_tests_assert_success, run_tests_in_session, TestCase


def test_console_format(suite, suite_test, config_override, tmpdir):
    config_override('log.format', 'file: {record.message}')
    config_override('log.console_format', 'console: {record.message}')
    config_override('log.root', str(tmpdir))
    suite_test.append_line('slash.logger.error("message here")')
    summary = suite.run(additional_args=['-vvv'])

    assert 'console: message here' in summary.get_console_output()

    [result] = summary.get_all_results_for_test(suite_test)
    with open(result.get_log_path()) as f:
        assert 'file: message here' in f.read()


def test_last_session_symlinks(files_dir, links_dir, session):

    test_log_file = files_dir.join(
        session.id, list(session.results.iter_test_results())[-1].test_metadata.id, "debug.log")
    assert test_log_file.check()
    session_log_file = files_dir.join(session.id, "session.log")
    assert session_log_file.check()

    assert links_dir.join("last-session").readlink() == session_log_file
    assert links_dir.join("last-session-dir").readlink() == session_log_file.dirname
    assert links_dir.join("last-test").readlink() == test_log_file


def test_global_result_get_log_path(files_dir, suite):
    summary = suite.run()
    assert summary.session.results.global_result.get_log_path() is not None
    assert summary.session.results.global_result.get_log_path().startswith(str(files_dir))


def test_log_file_colorize(files_dir, config_override, suite, suite_test):
    config_override('log.colorize', True)
    suite_test.append_line('slash.logger.notice("hey")')
    summary = suite.run()
    logfiles = [
        summary.session.results.global_result.get_log_path(),
        summary.get_all_results_for_test(suite_test)[0].get_log_path(),
    ]
    for logfile in logfiles:
        with open(logfile, 'rb') as f:
            log_data = f.read()

        assert b'\x1b[' in log_data


@pytest.mark.parametrize('level', ['info', 'notice', 'warning'])
def test_console_truncation_does_not_truncate_files(files_dir, suite, suite_test, config_override, level):
    assert slash.config.root.log.truncate_console_lines

    long_string = 'a' * 1000
    suite_test.append_line('slash.logger.{level}({msg!r})'.format(msg=long_string, level=level))
    summary = suite.run()
    [result] = summary.get_all_results_for_test(suite_test)
    with open(result.get_log_path()) as logfile:
        logfile_data = logfile.read()
        assert long_string in logfile_data


@pytest.mark.parametrize('symlink_name', ['last_session_symlink', 'last_session_dir_symlink', 'last_failed_symlink'])
def test_log_symlinks_without_root_path(suite, config_override, symlink_name):
    config_override('log.{0}'.format(symlink_name), 'some/subdir')
    assert suite.run().ok()


def test_last_test_not_overriden_by_stop_on_error(links_dir, suite):
    failed_test = suite[4]
    failed_test.when_run.fail()
    # we stop on error...
    for test in suite[5:]:
        test.expect_not_run()
    summary = suite.run(additional_args=['-x'])

    [failed_result] = summary.get_all_results_for_test(failed_test)

    for link_name in ('last-test', 'last-failed'):
        assert links_dir.join(link_name).readlink() == failed_result.get_log_path()


def test_last_test_delete_log_file(links_dir, suite, suite_test):
    os.makedirs(str(links_dir))
    temp_file = os.path.abspath(str(links_dir.join('somepath')))
    with open(temp_file, 'w'):
        pass
    os.symlink(temp_file, str(links_dir.join('last-test')))
    os.unlink(temp_file)

    assert not os.path.exists(str(links_dir.join('last-test')))
    assert os.path.islink(str(links_dir.join('last-test')))

    summary = suite.run()
    with open(summary.session.results.global_result.get_log_path()) as f:
        assert 'OSError: ' not in f.read()
    # assert links_dir.join('last-test').readlink() == list(summary.session.results)[-1].get_log_path()


def test_result_log_links(files_dir, session):

    for result in session.results.iter_test_results():
        assert result.get_log_path() is not None
        assert result.get_log_path().startswith(str(files_dir))


def test_last_failed(suite, links_dir):
    suite[-5].when_run.fail()
    last_failed = suite[-2]
    last_failed.when_run.fail()
    summary = suite.run()

    [result] = summary.get_all_results_for_test(last_failed)
    fail_log = result.get_log_path()
    assert os.path.isfile(fail_log)
    assert links_dir.join('last-failed').readlink() == fail_log


def test_unified_session_log(suite, suite_test, config_override, tmpdir):
    marker = 'some_marker_here'
    config_override('log.unified_session_log', True)
    config_override('log.root', str(tmpdir))
    config_override('log.session_subpath', 'session.log')
    suite_test.append_line('slash.logger.info({0!r})'.format(marker))
    summary = suite.run()
    with tmpdir.join('session.log').open() as f:
        assert marker in f.read()


@pytest.fixture
def session():
    session = run_tests_assert_success(SampleTest)
    return session


@pytest.fixture
def files_dir(logs_dir):
    return logs_dir.join("files")


@pytest.fixture
def links_dir(logs_dir):
    return logs_dir.join("links")


@pytest.fixture
def logs_dir(request, config_override, tmpdir, relative_symlinks):
    returned = tmpdir.join('logs')
    config_override("log.root", str(returned.join("files")))
    config_override("log.last_session_symlink",
                    str("../links/last-session" if relative_symlinks else returned.join("links", "last-session")))
    config_override("log.last_session_dir_symlink",
                    str("../links/last-session-dir" if relative_symlinks else returned.join("links", "last-session-dir")))
    config_override("log.last_test_symlink",
                    str("../links/last-test" if relative_symlinks else returned.join("links", "last-test")))
    config_override("log.last_failed_symlink",
                    str("../links/last-failed" if relative_symlinks else returned.join("links", "last-failed")))

    return returned


@pytest.fixture(params=[True, False])
def relative_symlinks(request):
    return request.param


@pytest.fixture
def links_dir(logs_dir):
    return logs_dir.join("links")


@pytest.fixture
def files_dir(logs_dir):
    return logs_dir.join("files")


_TOKEN = "logging-test"
_SESSION_START_MARK = "session-start-mark"
_SESSION_END_MARK = "session-end-mark"

_silenced_logger = logbook.Logger("silenced_logger")


class LogFormattingTest(TestCase):

    def setUp(self):
        super(LogFormattingTest, self).setUp()
        self.log_path = self.get_new_path()
        self.override_config(
            "log.root", self.log_path
        )
        self.override_config(
            "log.format", "-- {record.message} --"
        )
        self.override_config("log.subpath", "debug.log")

    def test(self):
        self.session = run_tests_assert_success(SampleTest)
        with open(os.path.join(self.log_path, "debug.log")) as logfile:
            for line in logfile:
                self.assertTrue(line.startswith("-- "))
                self.assertTrue(line.endswith(" --\n"))


class LoggingTest(TestCase):

    def test(self):
        self.log_path = self.get_new_path()
        self.override_config(
            "log.root",
            self.log_path,
        )
        self.override_config(
            "log.subpath",
            os.path.join("{context.session.id}",
                         "{context.test.__slash__.test_index0:03}-{context.test_id}", "debug.log")
        )
        self.override_config(
            "log.session_subpath",
            os.path.join("{context.session.id}", "debug.log")
        )
        self.override_config(
            "log.silence_loggers",
            [_silenced_logger.name]
        )

        self.addCleanup(gossip.unregister_token, _TOKEN)
        slash.hooks.session_start.register(
            functools.partial(_mark, _SESSION_START_MARK), token=_TOKEN)

        slash.hooks.session_end.register(
            functools.partial(_mark, _SESSION_END_MARK), token=_TOKEN)
        self.addCleanup(gossip.unregister_token, _TOKEN)

        self.session = run_tests_assert_success(SampleTest)
        self.tests_metadata = [
            result.test_metadata for result in self.session.results.iter_test_results()]
        self._test_all_run()
        self._test_test_logs_written()
        self._test_session_logs()
        self._test_no_silenced_logger_records()

    def _test_all_run(self):
        methods = [
            method_name for method_name in dir(SampleTest)
            if method_name.startswith("test")
        ]
        self.assertTrue(methods)
        self.assertEquals(len(self.tests_metadata), len(methods))

    def _test_test_logs_written(self):
        for test_metadata in self.tests_metadata:
            test_dir = "{0:03}-{1}".format(test_metadata.test_index0, test_metadata.id)
            log_path = os.path.join(
                self.log_path, self.session.id, test_dir, "debug.log")
            with open(log_path) as f:
                data = f.read()
            for other_test in self.tests_metadata:
                if other_test.id != test_metadata.id:
                    self.assertNotIn(other_test.id, data)
            self.assertNotIn(_SESSION_START_MARK, data)
            self.assertNotIn(_SESSION_END_MARK, data)

    def _test_session_logs(self):
        with open(os.path.join(self.log_path, self.session.id, "debug.log")) as f:
            data = f.read()
        self.assertIn(_SESSION_START_MARK, data)
        self.assertIn(_SESSION_END_MARK, data)
        for test_id in (t.id for t in self.tests_metadata):
            self.assertNotIn(test_id, data)

    def _test_no_silenced_logger_records(self):
        for path, _, filenames in os.walk(self.log_path):
            for filename in filenames:
                assert filename.endswith(".log")
                filename = os.path.join(path, filename)
                with open(filename) as f:
                    assert _silenced_logger.name not in f.read(
                    ), "Silenced logs appear in log file {0}".format(filename)


class ExtraLoggersTest(TestCase):

    def setUp(self):
        super(ExtraLoggersTest, self).setUp()
        self.session = slash.Session()
        self.handler = logbook.TestHandler()
        self.addCleanup(slash.log.remove_all_extra_handlers)
        slash.log.add_log_handler(self.handler)

    def test(self):
        with self.session:
            run_tests_in_session(SampleTest, session=self.session)
        for test_result in self.session.results.iter_test_results():
            for record in self.handler.records:
                if test_result.test_id in record.message:
                    break
            else:
                self.fail(
                    "Test id {} does not appear in logger".format(test_result.test_id))


class SampleTest(slash.Test):

    def test_1(self):
        _mark()

    def test_2(self):
        _silenced_logger.error("error")
        _silenced_logger.info("info")
        _silenced_logger.debug("debug")
        _mark()


def _mark(text=None):
    if text is None:
        text = slash.context.test_id
    slash.logger.debug(text)


class TestLocaltimeLogging(TestCase):

    def setUp(self):
        super(TestLocaltimeLogging, self).setUp()
        self.assertFalse(slash.config.root.log.localtime)
        self.path = self.get_new_path()
        self.override_config(
            "log.localtime", True)
        self.override_config(
            "log.root", self.path)

    def test_local_time(self):
        with slash.Session() as s:
            slash.logger.info("Hello")
        self.assertNotEquals(os.listdir(self.path), [])



import pytest
import slash
from slash.loader import Loader

from .utils import make_runnable_tests

@pytest.mark.parametrize('requirement_fullfilled', [True, False])
@pytest.mark.parametrize('use_message', [True, False])
@pytest.mark.parametrize('use_fixtures', [True, False])
def test_requirements(suite, suite_test, requirement_fullfilled, use_fixtures, use_message):

    suite_test.add_decorator('slash.requires(lambda: {0})'.format(requirement_fullfilled))
    if not requirement_fullfilled:
        suite_test.expect_skip()

    if use_fixtures:
        suite_test.depend_on_fixture(
            suite.slashconf.add_fixture())
    results = suite.run()
    if requirement_fullfilled:
        assert results[suite_test].is_success()
    else:
        assert not results[suite_test].is_started()
        assert results[suite_test].is_skip()


def test_requirements_on_class():

    def req1():
        pass

    def req2():
        pass

    @slash.requires(req1)
    class Test(slash.Test):

        @slash.requires(req2)
        def test_something(self):
            pass

    with slash.Session():
        [test] = make_runnable_tests(Test)

    assert [r._req for r in test.get_requirements()] == [req1, req2]



import operator

## utility operators

def safe_isinstance(a, b):
    try:
        return isinstance(a, b)
    except TypeError:
        return False

def safe_not_isinstance(a, b):
    return not safe_isinstance(a, b)

def is_none(x):
    return x is None

def is_not_none(x):
    return x is not None

def not_contains(x, y):
    """
    Python's operator module does not have a function for the ``not in`` operator. That's a shame.
    """
    return y not in x

def is_empty(x):
    return len(x) == 0

def is_not_empty(x):
    return not is_empty(x)

### Operator information
_OPERATORS = {}

def _op(operator_func, operator_token, inverse_func, inverse_token):
    _OPERATORS[operator_func] = Operator(operator_token, operator_func, inverse_func)
    _OPERATORS[inverse_func] = Operator(inverse_token, inverse_func, operator_func)

class Operator(object):
    def __init__(self, template, func, inverse_func):
        super(Operator, self).__init__()
        self.template = template
        self.func = func
        self.inverse_func = inverse_func

    def __call__(self, *args, **kwargs):
        return self.func(*args, **kwargs)

    def to_expression(self, *args, **kwargs):
        return self.template.format(*args, **kwargs)

def get_operator_by_func(operator_func):
    return _OPERATORS[operator_func]

### operator declarations
_op(operator.eq, "{0} == {1}", operator.ne, "{0} != {1}")
_op(safe_isinstance, "{0} is instance of {1}", safe_not_isinstance, "{0} is not instance of {1}")
_op(is_none, "{0} is None", is_not_none, "{0} is not None")
_op(operator.is_, "{0} is {1}", operator.is_not, "{0} is not {1}")
_op(operator.truth, "{0} is not false", operator.not_, "{0} is not true")
_op(operator.contains, "{1} in {0}", not_contains, "{1} not in {0}")
_op(is_empty, "{0} is empty", is_not_empty, "{0} is not empty")



import gossip
import slash

from .utils import TestCase


class GlobalStorageTest(TestCase):
    hook_called = False
    token = object()
    def test_global_storage_exists_on_session_start(self):
        @slash.exception_handling.disable_exception_swallowing
        def _on_session_start():
            self.assertIsNotNone(slash.g)
            slash.g.value = "value"
            self.assertEquals(slash.g.value, "value")
            self.hook_called = True
        slash.hooks.session_start.register(_on_session_start, token=self.token)
        self.addCleanup(
            gossip.unregister_token,
            self.token
        )
        with slash.Session() as s:
            with s.get_started_context():
                pass
        self.assertTrue(self.hook_called)



def mark_exception(e, name, value):
    """
    Associates a mark with a given value to the exception ``e``
    """
    _ensure_exception_marks(e)[name] = value


def get_exception_mark(e, name, default=None):
    """
    Given an exception and a label name, get the value associated with that mark label.
    If the label does not exist on the specified exception, ``default`` is returned.
    """
    return _ensure_exception_marks(e).get(name, default)


def _ensure_exception_marks(e):
    returned = getattr(e, "__slash_exc_marks__", None)
    if returned is None:
        returned = e.__slash_exc_marks__ = {}
    return returned

class ExceptionMarker(object):

    def __init__(self, name):
        super(ExceptionMarker, self).__init__()
        self.name = name

    def mark_exception(self, e):
        mark_exception(e, self.name, True)

    def is_exception_marked(self, e):
        return bool(get_exception_mark(e, self.name, False))



# -*- coding: utf-8 -*-
"""
slash.ext
~~~~~~~~~

Redirect imports from slash.ext.X to slash_X.

This code is an adaptation of flask.ext, copyright 2011 Armin Ronacher, licensed under BSD
"""


def setup():
    from ..exthook import ExtensionImporter
    importer = ExtensionImporter(['slash_%s'], __name__)
    importer.install()

setup()
del setup



import itertools
import json
import os

import logbook

from .utils.path import ensure_directory

_logger = logbook.Logger(__name__)

_RESUME_DIR = os.path.expanduser("~/.slash/session_states")
_RESUME_COUNTER = itertools.count()
_MAX_NUM_SAVED_SESSIONS = 10

LATEST = object()


def save_resume_state(session_result):
    resume_filename = _generate_resume_filename(session_result.session.id)
    tmp_filename = resume_filename + ".tmp"
    ensure_directory(os.path.dirname(tmp_filename))
    with open(tmp_filename, "w") as f:
        json.dump({
            "tests": [
                {"address": str(result.test_metadata.address), "needs_rerun":
                 result.is_failure() or result.is_error() or not result.is_started()}
                for result in session_result.iter_test_results()
            ],
        }, f)
    os.rename(tmp_filename, resume_filename)
    _logger.debug('Saved resume state to {0}', resume_filename)
    _cleanup_old_files()


def get_last_resumeable_session_id():
    files = _get_resume_state_files_by_mtime()
    if not files:
        raise CannotResume("No resume files found")
    return _get_session_id_from_filename(files[0][1])

def _get_session_id_from_filename(filename):
    return os.path.basename(filename).split("_", 1)[1].rsplit(".", 1)[0]


def _cleanup_old_files():
    for _, deleted_filename in _get_resume_state_files_by_mtime()[_MAX_NUM_SAVED_SESSIONS:]:
        _logger.debug('Deleting old statefile {0!r}...', deleted_filename)
        os.unlink(deleted_filename)

def _get_resume_state_files_by_mtime():
    return sorted(
        ((os.stat(f).st_mtime, f) for f in
         [os.path.join(_RESUME_DIR, f) for f in os.listdir(_RESUME_DIR)]),
        reverse=True)


def get_tests_to_resume(session_id):
    resume_filename = _find_resume_file_by_session_id(session_id)
    try:
        with open(resume_filename) as f:
            state = json.load(f)

    except (IOError, OSError) as e:
        raise CannotResume(
            "Cannot resume session {0} ({1})".format(session_id, e))

    return [test["address"] for test in state["tests"] if test["needs_rerun"]]

def _generate_resume_filename(session_id):
    return os.path.join(_RESUME_DIR, "{0:03}_{1}".format(next(_RESUME_COUNTER), session_id)) + ".json"

def _find_resume_file_by_session_id(session_id):
    for _, filename in _get_resume_state_files_by_mtime():
        if _get_session_id_from_filename(filename) == session_id:
            return filename
    raise CannotResume("Could not find resume file for session {0}".format(session_id))

class CannotResume(Exception):
    pass



import collections
import itertools
import random
from contextlib import contextmanager
from uuid import uuid1

import pytest
import slash
from slash._compat import iteritems, StringIO
from slash.core.fixtures.fixture_store import FixtureStore
from slash.core.fixtures.utils import get_scope_by_name, get_scope_name_by_scope

from .utils.code_formatter import CodeFormatter


def test_fixture_scopes(fixture_tree):
    with fixture_tree.testing_scope('session'):
        for i in range(3):
            with fixture_tree.testing_scope('module'):
                for i in range(5):
                    with fixture_tree.testing_scope('test'):
                        fixture_tree.check_values()

Structure = collections.namedtuple(
    'Structure', ['required', 'graph', 'scopes'])

_FLAT_STRUCTURE = Structure(
    required=[1, 2, 3],
    graph={1: [], 2: [], 3: []},
    scopes={},
)

_FLAT_STRUCTURE_WITH_UNRELATED = Structure(
    required=[1, 2, 3],
    graph={1: [], 2: [], 3: [], 4: []},
    scopes={},
)

_DEPENDENT_STRUCTURE = Structure(
    required=[1, 2, 3],
    graph={1: [3], 2: [1], 3: []},
    scopes={},
)

_DEPENDENT_STRUCTURE_WITH_UNRELATED = Structure(
    required=[1, 2, 3],
    graph={1: [3], 2: [1], 3: [], 4: [2], 5: [1]},
    scopes={},
)

_MODULE_SCOPED = Structure(
    required=[1, 2, 3],
    graph={1: [], 2: [], 3: []},
    scopes={1: 'module', 2: 'module'},
)

_DEPENDENT_MIXED_SCOPES = Structure(
    required=[1, 2, 3],
    graph={1: [2], 2: [3], 3: []},
    scopes={3: 'session', 2: 'module'},
)


@pytest.fixture(params=[
    _FLAT_STRUCTURE,
    _FLAT_STRUCTURE_WITH_UNRELATED,
    _DEPENDENT_STRUCTURE,
    _DEPENDENT_STRUCTURE_WITH_UNRELATED,
    _MODULE_SCOPED,
    _DEPENDENT_MIXED_SCOPES,
])
def structure(request):
    return request.param


@pytest.fixture
def fixture_tree(fixture_store, structure):
    returned = FixtureTree(fixture_store, structure)
    return returned


@pytest.fixture
def fixture_store():
    return FixtureStore()


class FixtureTree(object):

    def __init__(self, fixture_store, structure):
        super(FixtureTree, self).__init__()
        self._structure = structure
        self._cleanups_made = set()
        self._fixture_store = fixture_store
        self._fixtures = {}
        self._fixture_namegen = ('fixture_{0:05}'.format(x)
                                 for x in itertools.count(1000))
        self._required_names = []
        self._populate_fixtures()
        self._values = {}

    def check_values(self):
        values = self._fixture_store.get_fixture_dict(self._required_names)
        for required_name in self._required_names:
            assert values[required_name] is not None
            expected_value = self._values[required_name]
            assert values[required_name] == expected_value

        assert not (set(self._fixtures) - set(self._required_names)
                    ).intersection(self._values), 'Non-necessary fixtures unexpectedly initialized!'

    def check_value(self, name, value):
        assert self._values[name] == value

    def make_value(self, name):
        assert name not in self._values, 'Fixture generated more than once! (scope={0})'.format(
            get_scope_name_by_scope(self._fixtures[name].__slash_fixture__.scope))
        value = str(uuid1())
        self._values[name] = value
        return value

    def cleanup(self, name):
        assert name not in self._cleanups_made
        self._cleanups_made.add(name)

    @contextmanager
    def testing_scope(self, scope):
        self._fixture_store.push_scope(scope)
        yield
        self._fixture_store.pop_scope(scope, in_failure=False, in_interruption=False)
        scope_id = get_scope_by_name(scope)
        for fixture_name, fixture in iteritems(self._fixtures):
            if fixture.__slash_fixture__.scope <= scope_id:
                if fixture_name in self._values:
                    assert fixture_name in self._cleanups_made
                    self._cleanups_made.remove(fixture_name)
                    self._values.pop(fixture_name)
        assert not self._cleanups_made, 'Unknown cleanups called'

    def _populate_fixtures(self):
        assert not self._fixtures

        graph = self._structure.graph

        key_to_fixture_name = dict((key, next(self._fixture_namegen))
                                   for key in graph)

        stack = list(self._structure.graph)

        while stack:
            fixture_key = stack.pop()
            dependent_keys = graph[fixture_key]
            unresolved = [k for k in dependent_keys if key_to_fixture_name[k]
                          not in self._fixtures]
            if unresolved:
                stack.append(fixture_key)
                stack.extend(unresolved)
                continue

            fixture_name = key_to_fixture_name[fixture_key]
            if fixture_name in self._fixtures:
                continue

            fixture = self._fixtures[fixture_name] = self._construct_fixture(
                fixture_name,
                scope=self._structure.scopes.get(fixture_key, 'test'),
                dependent_names=[key_to_fixture_name[k] for k in dependent_keys])
            self._fixture_store.add_fixture(fixture)

        self._fixture_store.resolve()
        self._required_names.extend(key_to_fixture_name[k]
                                    for k in self._structure.required)

    def _construct_fixture(self, name, scope, dependent_names):
        buff = StringIO()
        code = CodeFormatter(buff)
        code.writeln(
            'def {0}(this, {1}):'.format(name, ', '.join(dependent_names)))
        with code.indented():
            for dependent_name in dependent_names:
                code.writeln(
                    'tree.check_value({0!r}, {0})'.format(dependent_name))
            code.writeln('@this.add_cleanup')
            code.writeln('def cleanup():')
            with code.indented():
                code.writeln('tree.cleanup({0!r})'.format(name))
            code.writeln('return tree.make_value({0!r})'.format(name))
        globs = {'tree': self}
        exec(buff.getvalue(), globs)
        return slash.fixture(scope=scope)(globs[name])
